﻿
-- TODO:
-- track data for login/out
-- report data for login logout
-- setup configurations on who to track / report
-- 

local time  = time

local FriendTracker = LibStub("AceAddon-3.0"):NewAddon("FriendTracker", "AceConsole-3.0", "AceBucket-3.0", "AceEvent-3.0", "AceHook-3.0")

local defaultDB = {
    factionrealm = {
        friends = {},
    },
    char = {
        frameLocation = {
            point="CENTER",
            relativePoint="CENTER",
            xOfs=0,
            yOfs=0,
            width=350,
            height=200,
            visible=true,
        }
    }
}

local options = {
    name = "FriendTracker",
    handler = FriendTracker,
    type = 'group',
    args = {
        reset = {
            type = 'execute',
            name = 'Reset Database',
            desc = 'Resets Friend Tracker to first time install state.',
            func = function() FriendTracker.db:ResetDB() end,
        },
        show = {
            type = 'execute',
            name = 'Show Friend Tracker',
            desc = 'Displays the Friend Tracker UI',
            func = 'ShowFrame',
        },
    }
}

function FriendTracker:OnInitialize()
	FriendTracker:Print("Friend Tracker Initialized")

	--initialize options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("FriendTracker", options, {"friendtracker", "ft"})
	
    FriendTracker.AceGUI = LibStub("AceGUI-3.0")
    
    FriendTracker.db = LibStub("AceDB-3.0"):New("FriendTrackerDB", defaultDB, true)
end


function FriendTracker:OnEnable()
    LibStub("DSxFriendEventLib"):Embed(self)

    FriendTracker:InitFrame()
    
    local baseFrame = CreateFrame("Frame", "FriendTrackerFrame", UIParent)
    FriendTracker:HookScript(baseFrame, "OnUpdate")

    if FriendTracker.db.char.frameLocation.visible then
        FriendTracker:ShowFrame()
    end
end

function FriendTracker:OnDisable()
    
end

function FriendTracker:OnUpdate()
    if not FriendTracker.lastUpdate or FriendTracker.lastUpdate < time() then
        FriendTracker:RedrawFrame()
    end
end


function FriendTracker:OnFriendLogin(record)
    local friend = self.db.factionrealm.friends[record.name]
    
    if friend == nil then
        friend={ events={} }
        self.db.factionrealm.friends[record.name] = friend
    end

    for k,v in pairs(record) do
        friend[k]=v
    end
    friend.connectTime=record.createTime
    friend.endTime=nil
    friend.seen=nil
end

function FriendTracker:OnFriendLocationChange(oldRecord, newRecord)
    local friend = self.db.factionrealm.friends[oldRecord.name]
    
    local event = {}
    event.location=oldRecord.location
    event.createTime=oldRecord.createTime
    event.endTime=oldRecord.endTime
    table.insert(friend.events, event)
    
    friend.location=newRecord.location
    friend.createTime=newRecord.createTime
end

function FriendTracker:OnFriendLogout(record)
    local friend = self.db.factionrealm.friends[record.name]

    local event = {}
    event.location=record.location
    event.createTime=record.createTime
    event.endTime=record.endTime
    table.insert(friend.events, event)
    
    friend.connected=nil
    friend.seen=nil
    friend.endTime=record.endTime
    friend.connectTime=record.endTime
end

--
--  GUI SECTION
--

function FriendTracker:InitFrame()
    if FriendTracker.frame == nil then
        FriendTracker.frame = FriendTracker.AceGUI:Create("Window")
        FriendTracker.frame:SetTitle("FriendTracker")
        FriendTracker.frame.frame:SetFrameStrata("LOW")
        FriendTracker.frame.frame:SetToplevel(false)
        FriendTracker.frame:SetLayout("Fill")
        
        --hook the onclose event from the close button and handle it ourselves
        FriendTracker:HookScript(FriendTracker.frame.closebutton, "OnClick", FriendTracker.OnCloseButtonPressed)
        
        FriendTracker:SecureHook(FriendTracker.frame.frame, "StopMovingOrSizing", FriendTracker.StopMovingOrSizing)
        
        local scrollFrame = FriendTracker.AceGUI:Create("ScrollFrame")
        scrollFrame:SetLayout("List")
        FriendTracker.frame:AddChild(scrollFrame)
        FriendTracker.container=scrollFrame
        
        FriendTracker.frame.frame:SetMinResize(200,100)
        
        local info = FriendTracker.db.char.frameLocation
        
        FriendTracker.frame.frame:SetPoint(info.point, nil, info.relativePoint, info.xOfs, info.yOfs)
        
        FriendTracker.frame.frame:SetWidth(info.width)
        FriendTracker.frame.frame:SetHeight(info.height)
        
        FriendTracker.frame:Hide()
    end
end

function FriendTracker.StopMovingOrSizing()
    local point, relativeTo, relativePoint, xOfs, yOfs = FriendTracker.frame.frame:GetPoint(1)

    FriendTracker.db.char.frameLocation.relativeTo=relativeTo
    FriendTracker.db.char.frameLocation.relativePoint=relativePoint
    FriendTracker.db.char.frameLocation.xOfs=xOfs
    FriendTracker.db.char.frameLocation.yOfs=yOfs
    FriendTracker.db.char.frameLocation.width=FriendTracker.frame.frame:GetWidth()
    FriendTracker.db.char.frameLocation.height=FriendTracker.frame.frame:GetHeight()
end

function FriendTracker.OnCloseButtonPressed()
    FriendTracker.frame:Hide()
    FriendTracker.db.char.frameLocation.visible=false
end

function FriendTracker:ShowFrame()
    FriendTracker.frame:Show()
    FriendTracker.db.char.frameLocation.visible=true
    
    FriendTracker:RedrawFrame()
end

local friends = {}
function FriendTracker:RedrawFrame()
    FriendTracker.lastUpdate=time()

    --clean out children from the container
    FriendTracker.container:ReleaseChildren()
    
    --read activity info into names
    
    while #friends > 0 do table.remove(friends, #friends) end
    
    for _, v in pairs(FriendTracker.db.factionrealm.friends) do
        if not v.seen then
            table.insert(friends, v)
        end
    end
    
    table.sort(friends, function (a, b) return a.connectTime > b.connectTime; end)
    
    for i=1,#friends do
        local friend = friends[i]
        
        local item = FriendTracker.AceGUI:Create("LineItem_FT")
        item:SetText(string.format(
            "%s - %s - %s", 
            friend.name,
            friend.location,
            FriendTracker:FormatTimeDelta(time()-friend.connectTime)
        ))
        
        item:SetWidth(220)
        item:SetPoint("LEFT", 0)
        item:SetPoint("RIGHT", 0)
        item:SetHeight(10)
        item:SetCallback("OnClick", function (data, event, button) friend.seen=1 end)
        
        if friend.connected then
            item:SetColor(0,1,0)
        else
            item:SetColor(1,0,0)
        end
        
        FriendTracker.container:AddChild(item)
    end
end

function FriendTracker:FormatTimeDelta(delta)
    local day=0
    local hour=0
    local minute=0
    local second=0;
    
    --days
    while delta > 60 * 60 * 24 do
        day = day + 1
        delta = delta - 60 * 60 * 24
    end
    --hours
    while delta > 60 * 60 do
        hour = hour + 1
        delta = delta - 60 * 60
    end
    --minutes
    while delta > 60 do
        minute = minute + 1
        delta = delta - 60
    end
    --second
    second=delta
    
    local ret =''
    
    if day > 0 then
        ret = ret .. day .. 'd'
    end
    if hour > 0 then
        ret = ret .. hour .. 'h'
    end
    if minute > 0 then
        ret = ret .. minute .. 'm'
    end
    if second > 0 then
        ret = ret .. second .. 's'
    end
    
    return ret
end












