﻿--[[-----------------------------------------------------------------------------
FriendTracker LineItem_FT widget
-------------------------------------------------------------------------------]]
local Type, Version = "LineItem_FT", 20
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local pairs = pairs

-- WoW APIs
local _G = _G
local PlaySound, CreateFrame, UIParent = PlaySound, CreateFrame, UIParent

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function Frame_OnClick(frame, ...)
	PlaySound("igMainMenuOption")
	frame.obj:Fire("OnClick", ...)
	AceGUI:ClearFocus()
end

local function Frame_OnEnter(frame)
	frame.obj:Fire("OnEnter")
end

local function Frame_OnLeave(frame)
	frame.obj:Fire("OnLeave")
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	["OnAcquire"] = function(self)
		-- restore default values
		self:SetHeight(24)
		self:SetWidth(200)
		self.label:SetText()
        self.label:SetTextColor(color,1,1,1)
	end,

	-- ["OnRelease"] = nil,

	["SetText"] = function(self, text)
		self.label:SetText(text)
	end,

	["SetColor"] = function(self, r, g, b)
		if not (r and g and b) then
			r, g, b = 1, 1, 1
		end
		self.label:SetVertexColor(r, g, b)
	end,
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local FrameBackdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 5, bottom = 3 }
}

local function Constructor()
	local name = "LineItem_FT" .. AceGUI:GetNextWidgetNum(Type)
	local frame = CreateFrame("Button", nil, UIParent)
    frame:ClearAllPoints()
    frame:SetPoint("TOPLEFT", 0,0)
    frame:SetPoint("BOTTOMRIGHT", 0,0)
    frame:Hide()

	frame:SetScript("OnClick", Frame_OnClick)
	frame:SetScript("OnEnter", Frame_OnEnter)
	frame:SetScript("OnLeave", Frame_OnLeave)

	local label = frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlightSmall")
    label:ClearAllPoints()
    label:SetPoint("TOPLEFT", 0,0)
    label:SetPoint("BOTTOMRIGHT", 0,0)
	label:SetJustifyH("LEFT")
	label:SetJustifyV("TOP")

	-- create widget
	local widget = {
		label = label,
		frame = frame,
		type  = Type
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end

	return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
